@@ -37,10 +37,18 @@ pub trait Token: Clone + fmt::Debug + Send + fmt::Display + Into<String> {
 ///
 /// 1. get an authentication token to use when accessing services,
 /// 2. get an endpoint URL for the given service type.
-pub trait Method: Clone + Send {
+pub trait Method: Sized {
     /// A token type.
     type TokenType: Token;
 
+    /// Default endpoint interface that is used when none is provided.
+    fn default_endpoint_interface(&self) -> String {
+        String::from("public")
+    }
+
+    /// Default region to use with this authentication (if any).
+    fn default_region(&self) -> Option<String> { None }
+
     /// Verify authentication and generate an auth token.
     fn get_token(&self, client: &Client) -> ApiResult<Self::TokenType>;
 
