@@ -14,10 +14,10 @@
 
 //! Generic API bits for implementing new services.
 
-use std::collections::HashMap;
 use std::marker::PhantomData;
 
 use hyper::{Get, Url};
+use hyper::header::Headers;
 use hyper::method::Method;
 use serde::Deserialize;
 use serde_json;
@@ -29,7 +29,8 @@ use super::utils;
 
 
 /// Type of query parameters.
-pub type Query = HashMap<String, String>;
+#[derive(Clone, Debug)]
+pub struct Query(pub Vec<(String, String)>);
 
 /// Information about API endpoint.
 #[derive(Clone, Debug)]
@@ -52,6 +53,12 @@ pub trait ServiceType {
         -> ApiResult<ServiceInfo>;
 }
 
+/// Trait representing a service with API version support.
+pub trait ApiVersioning {
+    /// Return headers to set for this API version.
+    fn api_version_headers(version: ApiVersion) -> ApiResult<Headers>;
+}
+
 /// A service-specific wrapper around Session.
 #[derive(Debug)]
 pub struct ServiceWrapper<'session, Auth: AuthMethod + 'session,
@@ -61,6 +68,19 @@ pub struct ServiceWrapper<'session, Auth: AuthMethod + 'session,
 }
 
 
+impl Query {
+    /// Empty query.
+    pub fn new() -> Query {
+        Query(Vec::new())
+    }
+
+    /// Add items to the query.
+    pub fn push<K, V>(&mut self, param: K, value: V)
+            where K: Into<String>, V: ToString {
+        self.0.push((param.into(), value.to_string()))
+    }
+}
+
 impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
         ServiceWrapper<'session, Auth, Srv> {
     /// Create a new wrapper for the specific service.
@@ -77,7 +97,7 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
             where P: IntoIterator, P::Item: AsRef<str> {
         let info = try!(self.session.get_service_info::<Srv>());
         let mut url = utils::url::extend(info.root_url, path);
-        let _ = url.query_pairs_mut().extend_pairs(query);
+        let _ = url.query_pairs_mut().extend_pairs(query.0);
         Ok(url)
     }
 
@@ -86,7 +106,10 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
             -> ApiResult<AuthenticatedRequestBuilder<'session, Auth>>
             where P: IntoIterator, P::Item: AsRef<str> {
         let url = try!(self.get_endpoint(path, query));
-        Ok(self.session.raw_request(method, url))
+        let headers = self.session.service_headers::<Srv>();
+        trace!("Sending HTTP {} request to {} with {:?}",
+               method, url, headers);
+        Ok(self.session.raw_request(method, url).headers(headers))
     }
 
     /// Make a GET request.
